#include <cstdio>
#include <algorithm>
#include <string>
#include <vector>
#include <map>
#include <set>

#include "testlib.h"

using namespace std;

typedef pair<int,int> pii;

const int COUNT[2] = {10, 8};
const int COUNT_BAD[2] = {100, 100};

struct MinTree {
private:
	int kk;
	vector <int> tree;
	void update(int t)
	{
		tree[t] = min(tree[2 * t], tree[2 * t + 1]);
	}
public:
	static const int INF = 1 << 30;

	MinTree (int size)
	{
		size += 10;
		kk = 1;
		while (kk < size)
			kk *= 2;

		tree = vector<int>(kk * 2, INF);
	}
	void modify(int t, int value)
	{
		t += kk;
		tree[t] = value;
		while (t > 1)
		{
			t /= 2;
			update(t);
		}
	}
	int getMin(int l, int r)
	{
		int ans = INF;
		l += kk;
		r += kk;
		while (l <= r)
		{
			if (l % 2 == 1)
			{
				ans = min(ans, tree[l]);
				++l;
			}
			if (r % 2 == 0)
			{
				ans = min(ans, tree[r]);
				--r;
			}
			l /= 2;
			r /= 2;
		}
		return ans;
	}
};

struct Segment {
	int l, r;

	Segment(int _l, int _r) : l(_l), r(_r) {}

	bool operator <= (const Segment & s) const
	{
		return s.l <= l && r <= s.r;
	}
};

struct Event {
	bool open;
	int num;
	int x;
	Segment seg;

	Event (bool _open, int _num, int _x, Segment _seg) : open(_open), num(_num), x(_x), seg(_seg) {}

	bool operator < (const Event & e) const
	{
		if (x != e.x)
			return x < e.x;

		if (open != e.open)
			return open > e.open;

		if (open)
			return seg.r > e.seg.r;
		else
			return seg.l > e.seg.l;
	}

	void print()
	{
		printf("x = %d num = %d open = %d\n", x, num, int(open));
	}
};

struct List {
private:
	struct ListElement {
		int num;
		int prev, next;
	};

	vector <ListElement> list;
	int used;
	int fake;
	vector <int> numInList;

public:

	List (int size)
	{
		++size;
		list.resize(size);
		numInList.resize(size, size);

		used = 0;
		fake = used++;

		list[fake].next = list[fake].prev = fake;
	}

	void insertBehind(int next, int num)
	{
		int prev = list[next].prev;

		list[prev].next = num;
		list[next].prev = num;

		list[num].next = next;
		list[num].prev = prev;
	}

	void deleteElement(int num)
	{
		int prev = list[num].prev;
		int next = list[num].next;

		list[prev].next = next;
		list[next].prev = prev;
	}

	int newElement(int num)
	{
		int cur = used++;
		list[cur].num = num;
		numInList[num] = cur;
		list[cur].next = list[cur].prev = -1;
		return cur;
	}

	void insertAtEnd(int num)
	{
		insertBehind(fake, num);
	}

	int getPrev(int num) const
	{
		return list[num].prev;
	}

	int getNumIn(int num) const
	{
		return numInList[num];
	}

	int getFake() const
	{
		return fake;
	}

	int getNum(int num) const
	{
		return list[num].num;
	}
};


void printTest(vector <pii> v, vector <int> pans, vector <int> jans, bool offline);

void genAnswers(vector <pii> v, bool offline)
{
	shuffle(v.begin(), v.end());

	random_t rnd = ::rnd;

	int n = int(v.size());

	vector <Segment> seg;
	seg.reserve(n);

	for (int i = 0; i < n; ++i)
		seg.push_back(Segment(v[i].first, v[i].second));

	vector <int> pans(n, -1), jans(n, -1);
	vector <bool> correct(n, false);

	MinTree minTree(n);

	List list(n);

	vector <Event> events;

	for (int i = 0; i < n; ++i)
	{
		events.push_back(Event(true, i, seg[i].l, seg[i]));
		events.push_back(Event(false, i, seg[i].r, seg[i]));
	}

	sort(events.begin(), events.end());

	for (int i = 0; i < int(events.size()); ++i)
	{
		int num = events[i].num;

		if (events[i].open)
		{
			int curNum = list.newElement(num);
			minTree.modify(curNum, seg[num].r);
			list.insertAtEnd(curNum);
		}
		else
		{
			int cur, prev;
			cur = list.getNumIn(num);
			prev = list.getPrev(cur);

			int variants[2] = {num, num};

			if (prev == list.getFake())
				jans[num] = -1;
			else
			{
				jans[num] = list.getNum(prev);

				if (offline)
					variants[true] = jans[num];
			}

			for (int j = 0, cntBad = 0; j < COUNT[offline] && cntBad < COUNT_BAD[offline]; ++j)
			{
				int ans;
				if (offline)
				{
					if (2 * j >= COUNT[offline])
						ans = rnd.next(0, n / 2 - 1);
					else
						ans = rnd.next(n / 2, n - 1);
				}
				else
					ans = rnd.next(0, n - 1);

				bool ok, bad;

				if (prev == list.getFake())
					bad = true;
				else
				{
					if (ans == -1 || !(seg[num] <= seg[ans]) || ans == num)
						bad = true;
					else
					{
						int rightAns = seg[ans].r;
						int minRight = minTree.getMin(list.getNumIn(ans) + 1, cur - 1);

						bad = false;
						ok = minRight > rightAns;
					}
				}

				if (bad)
				{
					--j, cntBad++;
				}
				else
				{
					if (list.getNumIn(variants[ok]) > list.getNumIn(ans))
						variants[ok] = ans;
				}
			}

			int action = rnd.next(0, 204);

			if (action <= 3)
			{
				pans[num] = rnd.next(-1, n - 1);
				correct[num] = false;
			}
			else if (action <= 4)
			{
				pans[num] = num;
				correct[num] = false;
			}
			else if (action <= 100)
			{
				pans[num] = variants[false];
				correct[num] = false;
			}
			else
			{
				pans[num] = variants[true];
				correct[num] = (pans[num] != num);
			}

			minTree.modify(cur, minTree.INF);
			list.deleteElement(cur);
		}
	}

/*	for (int i = 0; i < n; ++i)
		if (correct[i] && rnd.next(0, 10) == 0)
			swap(pans[i], jans[i]);*/
		
	printTest(v, pans, jans, offline);
}


int StartId;

void printTest(vector <pii> v, vector <int> pans, vector <int> jans, bool offline)
{
	static int id = StartId;
	static char filename[10];

#ifdef DBG1	
	sprintf(filename, "%02d", id);
#else
	sprintf(filename, "%d", id);
#endif
	++id;

	int n = int(v.size());

	random_t rnd = ::rnd;
	vector <int> perm(n);
	for (int i = 0; i < n; ++i)
		perm[i] = i;
	if (offline)
		for (int i = 1; i < n; ++i)
			swap(perm[i], perm[rnd.next(0, i - 1)]);

	vector <int> rev(n);
	for (int i = 0; i < n; ++i)
		rev[perm[i]] = i;

	for (int i = 0; i < n; ++i)
	{
		if (jans[i] != -1)
			jans[i] = rev[jans[i]];
		if (pans[i] != -1)
			pans[i] = rev[pans[i]];
	}

	fprintf(stderr, "%s\n", filename);

	FILE * fout = fopen(filename, "wt");

	fprintf(fout, "%d\n", int(v.size()));
	for (int ii = 0; ii < n; ++ii)
	{
		int i = perm[ii];
		if (v[i].second == v[i].first)
			v[i].second++;
		fprintf(fout, "%d %d\n", v[i].first, v[i].second);
	}
	for (int ii = 0; ii < n; ++ii)
	{
		int i = perm[ii];
		if (ii != 0)
			fprintf(fout, " ");
		fprintf(fout, "%d", pans[i] + 1);
	}
	fprintf(fout, "\n");
	for (int ii = 0; ii < n; ++ii)
	{
		int i = perm[ii];
		if (ii != 0)
			fprintf(fout, " ");
		fprintf(fout, "%d", jans[i] + 1);
	}
	fprintf(fout, "\n");

	fclose(fout);
}

void genAnswersWithNoise(vector <pii> v, bool offline)
{	
	int n = int(v.size());

	for (int i = 0; i < n / 100; ++i)
	{
		int j = rnd.next(0, n - 1);
		int k = rnd.next(0, n - 1);

		int arr[4] = {v[j].first, v[j].second, v[k].first, v[k].second};

		bool diff = 1;
		for (int ii = 0; ii < 4; ++ii)
			for (int jj = ii + 1; jj < 4; ++jj)
				diff &= arr[ii] != arr[jj];

		if (!diff)
			continue;

		while (1)
		{
			int ii = rnd.next(0, 1);
			int jj = rnd.next(2, 3);
			if (ii == 0)
			{
				if (arr[jj] < arr[1])
				{
					v[j].first = arr[jj];
					break;
				}
			}
			else
			{
				if (arr[0] < arr[jj])
				{
					v[j].second = arr[jj];
					break;
				}
			}
		}
	}

	genAnswers(v, offline);
}

void genRandom(int n, int amax, int amin, bool offline)
{
	vector<pii> v(n);

	for (int i = 0; i < n; ++i) {
		int l, r;
		do {
			l = rnd.next(amin, amax);
			r = rnd.next(amin, amax);
		} while(l == r);

		if (l > r)
			swap(l, r);

		v[i].first = l;
		v[i].second = r;
	}
	genAnswers(v, offline);
}

vector <int> genVectorWithDifferentNumber(int n, int amax, int amin)
{
	vector <int> v;
	set <int> nums;

	for (int i = 0; i < n; ++i)
	{
		int x;
		do {
			x = rnd.next(amin, amax);
		} while (nums.find(x) != nums.end());

		v.push_back(x);
		nums.insert(x);
	}

	sort(v.begin(), v.end());

	return v;
}

void genTest1(int type, int n, int amax, int amin, bool offline)
{
	vector<pii> v(n);

	int cnt1, cnt2;
	cnt1 = n / 2;
	cnt2 = n - cnt1;

	vector <int> x = genVectorWithDifferentNumber(2 * n, amax, amin);

	int l = 0;
	int r = 2 * n - cnt1;

	for (int i = 0; i < cnt1; ++i)
	{
		v[i].first = x[l++];
		v[i].second = x[r++];
	}

	if (type == 0)
	{
		for (int i = 0; i < cnt2; ++i)
		{
			v[i + cnt1].first = x[l + 2 * i];
			v[i + cnt1].second = x[l + 2 * i + 1];
		}
	}
	else
	{
		for (int i = 0; i < cnt2; ++i)
		{
			v[i + cnt1].first = x[l + i];
			v[i + cnt1].second = x[l + cnt2 + i];
		}
	}

	if (offline)
		genAnswers(v, offline);
	else
		genAnswersWithNoise(v, offline);
}

void genTest2(int n, int amax, int amin, bool offline)
{
	vector<pii> v(n);

	vector <int> x = genVectorWithDifferentNumber(2 * n, amax, amin);

	int l = 0;
	int r = 2 * n - 1;

	for (int i = 0; i < n; ++i)
	{
		v[i].first = x[l++];
		v[i].second = x[r--];
	}

	genAnswersWithNoise(v, offline);
}

void genTest3(int n, int amax, int amin, bool offline)
{
	vector<pii> v(n);

	vector <int> x = genVectorWithDifferentNumber(2 * n, amax, amin);

	int l = 0;
	int r = n / 2;

	for (int i = 0; i < n; ++i)
	{
		v[i].first = x[l++];
		v[i].second = x[r++];
	}

	genAnswersWithNoise(v, offline);
}

void genTest4(int n, int amax, int amin, bool offline)
{
	vector<pii> v(n);

	vector <int> x = genVectorWithDifferentNumber(2 * n, amax, amin);

	int l = 0;

	for (int i = 0; i < n; ++i)
	{
		v[i].first = x[l++];
		v[i].second = x[l++];
	}

	genAnswersWithNoise(v, offline);
}

void genTest5(int type, int n, int amax, int amin, bool offline)
{
	vector<pii> v(n);

	vector <int> x = genVectorWithDifferentNumber(n, amax, amin + 1);

	int left = rnd.next(amin, x[0]);

	for (int i = 0; i < n; ++i)
	{
		v[i].first = left;
		v[i].second = x[i];
	}

	if (type == 1)
	{
		for (int i = 0; i < n; ++i)
		{
			v[i].first = amax + amin - v[i].first;
			v[i].second = amax + amin - v[i].second;

			swap(v[i].first, v[i].second);
		}
	}

	genAnswers(v, offline);
}

bool onePossibleAnswer(int n, const vector <pii> & v)
{
	for (int j = 0; j < n - 1; ++j)
		if (v[j] == v[n - 1])
			return 0;

	for (int i = 0; i < n; ++i)
	{
		int cnt = 0;
		for (int j = 0; j < n; ++j)
		{
			if (j == i)
				continue;
			if (!(v[j].first <= v[i].first && v[i].second <= v[j].second))
				continue;

			int k = 0;

			while (k < n && (k == i || k == j || 
						!(v[j].first <= v[k].first && v[k].first <= v[i].first 
							&& v[i].second <= v[k].second && v[k].second <= v[j].second)))
				++k;

			if (k == n)
				++cnt;
		}

		if (cnt > 1)
			return 0;
	}

	return 1;
}

void genTest0(int n, int amax, int amin)
{
	vector <pii> v(n);

	for (int i = 0; i < n; ++i)
	{
		while (1)
		{
			int l = rnd.next(amin, amax);
			int r = rnd.next(amin, amax);

			if (l == r)
				continue;

			if (l > r)
				swap(l, r);

			v[i].first = l;
			v[i].second = r;

			if (onePossibleAnswer(i + 1, v))
				break;
		}
	}

	genAnswers(v, false);
}

void genAllTest(int n, int MAX)
{
	genRandom(n, MAX, -MAX, false);

	genTest1(0, n, MAX, -MAX, false);
	genTest1(1, n, MAX, -MAX, false);

	genTest2(n, MAX, -MAX, false);

	genTest3(n, MAX, -MAX, false);

	genTest4(n, MAX, -MAX, false);

	genTest5(0, n, MAX, -MAX, false);
	genTest5(1, n, MAX, -MAX, false);
}

void genAllOfflineTest(int n, int MAX)
{
	genRandom(n, MAX, -MAX, true);

	genTest1(0, n, MAX, -MAX, true);
	genTest1(1, n, MAX, -MAX, true);

	genTest2(n, MAX, -MAX, true);
	
}

int main(int argc, char * argv[])
{
	registerGen(argc, argv);
	StartId = atoi(argv[1]);
	int gen1st = atoi(argv[2]);
	int gen2nd = atoi(argv[3]);
	int gen3rd = atoi(argv[4]);

	if (gen1st) {
		fprintf(stderr, "group 1 generated...\n");
		const int testCnt = 5;
		const int testN[testCnt] = {
			10,
			25,
			50,
			75,
			100
		};

		const int MAX = 1000;

		for (int id = 0; id < testCnt; ++id)
			genTest0(testN[id], MAX, -MAX);
	}

	if (gen2nd) {
		fprintf(stderr, "group 2 generated...\n");
		const int testCnt = 2;
		const int testN[testCnt] = {
			5000,
			10000
		};

		const int MAX = int(1e9);

		for (int id = 0; id < testCnt; ++id)
		{
			int n = testN[id];

			genAllTest(n, MAX);
		}
	}

	if (gen3rd) {
		fprintf(stderr, "group 3 generated...\n");
		const int testCnt = 2;
		const int testN[testCnt] = {
			75000, 100000
		};

		const int MAX = int(1e9);

		for (int id = 0; id < testCnt; ++id)
			genAllOfflineTest(testN[id], MAX);
	}

	return 0;
}
